from bs4 import BeautifulSoup
import requests
from concurrent.futures import ThreadPoolExecutor

def get_stock(url):
    web = requests.get(url)
    soup = BeautifulSoup(web.text, "html.parser")
    error = None
    if web.status_code == 404:
        error = url + "/Error"
        return error
    company = soup.select('h1')[1].get_text()
    quote = soup.find('span', {'class': 'Fz(32px)'})
    difference = soup.find('span', {'class': 'Fz(20px)'}).get_text()

    up_or_down = ''
    if "C($c-trend-up)" in quote.attrs["class"]:
        up_or_down = '+'
    elif "C($c-trend-down)" in quote.attrs["class"]:
        up_or_down = '-'
    else : 
        up_or_down = '-'

    return (f'{ company } : { quote.get_text() } ( { up_or_down }{ difference } )')

def get_error(url):
    if "Error" in url:
        return url.split('/')[4]
    
def map_together(func, lst):
    data = []
    executor = ThreadPoolExecutor()
    with ThreadPoolExecutor() as executor:
        data = list(executor.map(func, lst))
    return data

def get_url(codes):
    url = 'https://tw.stock.yahoo.com/quote/'
    return f'{url}{codes}'

def get_value_from_request(request):
    lst = []
    for idx in request:
        stock_code = request.getlist(idx)[0]
        if stock_code != "":
            lst.append(f'{stock_code}')
    return lst



from flask import Flask, render_template, request, redirect, url_for
from crawlerHelper import get_stock, map_together, get_url, get_error, get_value_from_request
crawler_url = 'https://tw.stock.yahoo.com/quote/'

app = Flask(__name__)

app.config['DEBUG'] = True
stock_lst = []

@app.route("/stock/setting", methods=['POST', 'GET'])
def put_stock():
    if request.method == 'POST':
        stock_codes = get_value_from_request(request.form)
        stock_urls = map_together(get_url, stock_codes)
        stock_results = map_together(get_stock, stock_urls)
        error_codes = list(
            filter(None, map_together(get_error, stock_results)))
       
        if error_codes:
            return render_template('setStock.html', error=error_codes)
        return redirect(url_for('stock'))
    else:
        return render_template('setStock.html')

@app.route("/stock/result", methods=['POST'])
def get_stock_result():
    codes_storage = list(request.get_json().values()) 
    stock_urls = map_together(get_url, codes_storage)  
    data = map_together(get_stock, stock_urls)
    return data
    

@app.route("/stock")
def stock():
    return render_template('home.html')

if __name__ == "__main__":
    app.run(host='0.0.0.0', debug=True)



import setuptools
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="peekStock",
    version="0.0.1",
    author="Nancy Hsu",
    author_email="yutinghsu0411@gmail.com",
    description="A easy tool to track your stock on your work",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Nancy-Hsu/Peek-Stock/tree/main",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],  
)
